import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/* eslint-disable no-plusplus */
import React, { useMemo, useEffect, useRef, forwardRef, useImperativeHandle, useContext } from 'react';
import classnames from 'classnames';
import { CSSTransition } from 'react-transition-group';
import Icon from '../icon';
import Overlay from '../overlay';
import useEventListener from '../hooks/use-event-listener';
import { useLockScroll } from '../hooks/use-lock-scroll';
import { isDef } from '../utils';
import { callInterceptor } from '../utils/interceptor';
import { renderToContainer } from '../utils/dom/renderToContainer';
import useSsrCompat from '../hooks/use-ssr-compat';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import PopupContext from './PopupContext';
export var sharedPopupProps = ['round', 'zIndex', 'closeable', 'overlay', 'overlayClass', 'overlayStyle', 'destroyOnClose', 'mountOnEnter', 'lockScroll', 'duration', 'transition', 'closeOnClickOverlay', 'closeOnPopstate', 'onClickOverlay', 'safeAreaInsetBottom', 'onOpen', 'onClose', 'onOpened', 'onClosed', 'beforeClose'];
var Popup = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('popup', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var style = props.style,
      round = props.round,
      visible = props.visible,
      closeable = props.closeable,
      title = props.title,
      descrition = props.descrition,
      children = props.children,
      closeIcon = props.closeIcon,
      position = props.position,
      onClickOverlay = props.onClickOverlay,
      closeOnClickOverlay = props.closeOnClickOverlay,
      onOpen = props.onOpen,
      beforeClose = props.beforeClose,
      onClose = props.onClose,
      onClickCloseIcon = props.onClickCloseIcon,
      teleport = props.teleport;
  var opened = useRef(false);
  var popupRef = useRef();
  var popupZIndex = useRef(2000);

  var _useSsrCompat = useSsrCompat(),
      _useSsrCompat2 = _slicedToArray(_useSsrCompat, 2),
      ssrCompatRender = _useSsrCompat2[0],
      rendered = _useSsrCompat2[1];

  var popupStyle = useMemo(function () {
    var initStyle = _objectSpread({}, style);

    if (isDef(props.duration)) {
      var key = position === 'center' ? 'animationDuration' : 'transitionDuration';
      initStyle[key] = "".concat(props.duration, "ms");
    }

    return initStyle;
  }, [JSON.stringify(style), props.duration]);

  var open = function open() {
    if (!opened.current) {
      opened.current = true;
      onOpen === null || onOpen === void 0 ? void 0 : onOpen();
    }
  };

  var close = function close() {
    if (opened.current) {
      callInterceptor({
        interceptor: beforeClose,
        args: ['close'],
        done: function done() {
          opened.current = false;
          onClose === null || onClose === void 0 ? void 0 : onClose();
        }
      });
    }
  };

  var handleClickOverlay = function handleClickOverlay(event) {
    onClickOverlay === null || onClickOverlay === void 0 ? void 0 : onClickOverlay(event);

    if (closeOnClickOverlay) {
      close();
    }
  };

  var renderOverlay = function renderOverlay() {
    var overlay = props.overlay,
        overlayClass = props.overlayClass,
        overlayStyle = props.overlayStyle;

    if (overlay) {
      return /*#__PURE__*/React.createElement(Overlay, {
        visible: visible,
        className: overlayClass,
        customStyle: overlayStyle,
        zIndex: popupZIndex.current,
        duration: props.duration,
        onClick: handleClickOverlay
      });
    }

    return null;
  };

  var handleClickCloseIcon = function handleClickCloseIcon(e) {
    onClickCloseIcon === null || onClickCloseIcon === void 0 ? void 0 : onClickCloseIcon(e);
    close();
  };

  var renderCloseIcon = function renderCloseIcon() {
    if (closeable) {
      var closeIconPosition = props.closeIconPosition,
          iconPrefix = props.iconPrefix;

      if (closeIcon && typeof closeIcon === 'string') {
        return /*#__PURE__*/React.createElement(Icon, {
          name: closeIcon,
          className: classnames(bem('close-icon', closeIconPosition)),
          classPrefix: iconPrefix,
          onClick: handleClickCloseIcon
        });
      }

      return /*#__PURE__*/React.createElement("div", {
        className: classnames(bem('close-icon', closeIconPosition)),
        onClick: onClickCloseIcon
      }, closeIcon);
    }

    return null;
  };

  var renderTitle = function renderTitle() {
    if (title) {
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(bem('title'))
      }, title);
    }

    return null;
  };

  var renderDescrition = function renderDescrition() {
    if (descrition) {
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(bem('descrition'))
      }, descrition);
    }

    return null;
  };

  var handleClick = function handleClick(e) {
    var _props$onClick;

    (_props$onClick = props.onClick) === null || _props$onClick === void 0 ? void 0 : _props$onClick.call(props, e);
  };

  var renderPopup = function renderPopup() {
    return /*#__PURE__*/React.createElement("div", {
      ref: popupRef,
      style: _objectSpread(_objectSpread({
        zIndex: popupZIndex.current
      }, popupStyle), {}, {
        display: !visible ? 'none' : undefined
      }),
      className: classnames(props.className, bem(_defineProperty({
        round: round
      }, position, position)), {
        'rc-safe-area-bottom': props.safeAreaInsetBottom
      }),
      onClick: handleClick
    }, renderTitle(), renderDescrition(), children, renderCloseIcon());
  };

  var renderTransition = function renderTransition() {
    var transition = props.transition,
        destroyOnClose = props.destroyOnClose,
        duration = props.duration,
        mountOnEnter = props.mountOnEnter,
        onClosed = props.onClosed,
        onOpened = props.onOpened;
    var name = position === 'center' ? 'rc-fade' : "rc-popup-slide-".concat(position);
    return /*#__PURE__*/React.createElement(CSSTransition, {
      in: visible,
      appear: true,
      timeout: {
        exit: duration
      },
      classNames: transition || name,
      mountOnEnter: !mountOnEnter,
      unmountOnExit: destroyOnClose,
      onEnter: open,
      onEntered: onOpened,
      onExited: function onExited() {
        onClosed === null || onClosed === void 0 ? void 0 : onClosed();
      }
    }, renderPopup());
  };

  useEventListener('popstate', function () {
    if (props.closeOnPopstate) {
      close();
    }
  });
  useLockScroll(popupRef, props.visible);
  useEffect(function () {
    if (visible) {
      if (popupRef.current) {
        var _window$getComputedSt;

        var curIndex = +((_window$getComputedSt = window.getComputedStyle(popupRef.current, null)) === null || _window$getComputedSt === void 0 ? void 0 : _window$getComputedSt.zIndex);

        if (isDef(curIndex)) {
          popupZIndex.current++;
        }
      }
    }
  }, [visible]);
  useImperativeHandle(ref, function () {
    return {
      popupRef: popupRef
    };
  });
  return ssrCompatRender(function () {
    return renderToContainer(teleport, /*#__PURE__*/React.createElement(PopupContext.Provider, {
      value: {
        visible: visible
      }
    }, renderOverlay(), renderTransition()));
  });
});
Popup.defaultProps = {
  mountOnEnter: true,
  overlay: true,
  lockScroll: true,
  position: 'center',
  closeIcon: 'cross',
  closeIconPosition: 'top-right',
  closeOnClickOverlay: true,
  teleport: function teleport() {
    return document.body;
  }
};
Popup.displayName = 'Popup';
export default Popup;